@@ -26,17 +26,17 @@
  * IN THE SOFTWARE.
  */
 
-require("../conf/config.php");
-require("class.rFastTemplate.php");
+require(dirname(dirname(__FILE__))."/conf/config.php");
+require(dirname(__FILE__)."/class.rFastTemplate.php");
 
 function mlmmj_boolean($name, $nicename, $text) 
 {
     global $tpl, $topdir, $list;
 
     if(is_file($topdir."/".$list."/control/".$name))
-	$checked = TRUE;
+       $checked = TRUE;
     else
-	$checked = FALSE;
+       $checked = FALSE;
 
     $tpl->assign(array("NAME" => htmlentities($name),
 		       "NICENAME" => htmlentities($nicename),
@@ -53,12 +53,13 @@ function mlmmj_string($name, $nicename, $text)
     $file = $topdir."/".$list."/control/".$name;
     $value = "";
 
-    if(!is_file($file))
-	$lines = "";
-    else
-	$lines = file($file);
+    if(is_file($file)) {
+       $lines = file($file);
+       $value = $lines[0];
+    }
 
-    $value = $lines[0];
+    // remove trailing \n if any, just to be sure
+    $value = preg_replace('/\n$/',"",$value);
     
     $tpl->assign(array("NAME" => htmlentities($name),
 		       "NICENAME" => htmlentities($nicename),
@@ -75,15 +76,12 @@ function mlmmj_list($name, $nicename, $text)
     $file = "$topdir/$list/control/$name";
     $value = "";
 
-    if(!is_file($file))
-	$lines = array();
-    else
-	$lines = file($file);
+    if(is_file($file))
+       $value = file_get_contents($file);
 
-    foreach ($lines as $line) 
-    {
-	$value .= $line;
-    }
+    // the last \n would result in an extra empty line in the list box,
+    // so we remove it
+    $value = preg_replace('/\n$/',"",$value);
 
     $tpl->assign(array("NAME" => htmlentities($name),
 		       "NICENAME" => htmlentities($nicename),
@@ -99,11 +97,14 @@ function encode_entities($str) { return htmlentities($str); }
 
 $tpl = new rFastTemplate($templatedir);
 
-$list = $HTTP_GET_VARS["list"];
+$list = $_GET["list"];
 
 if(!isset($list))
 die("no list specified");
 
+if (dirname(realpath($topdir."/".$list)) != realpath($topdir))
+die("list outside topdir");
+
 if(!is_dir($topdir."/".$list))
 die("non-existent list");
 
@@ -114,10 +115,7 @@ $tpl->define(array("main" => "edit.html",
 
 $tpl->assign(array("LIST" =>htmlentities($list)));
 
-$handle = fopen("$templatedir/../conf/tunables.pl", "r");
-$tunables = fread($handle, filesize("$templatedir/../conf/tunables.pl"));
-fclose($handle);
-
+$tunables = file_get_contents($confdir.'/tunables.pl');
 eval($tunables);
 
 $tpl->parse("MAIN","main");
