@@ -1,6 +1,6 @@
 /* Copyright (C) 2002, 2003 Mads Martin Joergensen <mmj at mmj.dk>
  *
- * $Id: mlmmj-unsub.h,v 1.7 2005/10/09 14:17:52 mmj Exp $
+ * $Id$
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -24,14 +24,5 @@
 #ifndef MLMMJ_UNSUBSCRIBE_H
 #define MLMMJ_UNSUBSCRIBE_H
 
-#include <sys/types.h>
-
-void confirm_unsub(const char *listdir, const char *listaddr,
-		   const char *listdelim, const char *subaddr,
-		   const char *mlmmj, enum subtype typesub);
-ssize_t unsubscribe(int subreadfd, int subwritefd, const char *address);
-void generate_unsubconfirm(const char *listdir, const char *listaddr,
-			   const char *listdelim, const char *subaddr,
-			   const char *mlmmjsend, enum subtype typesub);
 
 #endif /* MLMMJ_UNSUBSCRIBE_H */
