@@ -1,6 +1,6 @@
 /* Copyright (C) 2003 Mads Martin Joergensen <mmj at mmj.dk>
  *
- * $Id: subscriberfuncs.h,v 1.5 2004/09/23 14:29:27 mmj Exp $
+ * $Id$
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -26,6 +26,6 @@
 
 off_t find_subscriber(int fd, const char *address);
 int is_subbed_in(const char *subddirname, const char *address);
-int is_subbed(const char *listdir, const char *address);
+enum subtype is_subbed(const char *listdir, const char *address, int both);
 
 #endif /* SUBSCRIBERFUNC_H */
