@@ -1,6 +1,6 @@
 /* Copyright (C) 2002, 2003 Mads Martin Joergensen <mmj at mmj.dk>
  *
- * $Id: checkwait_smtpreply.c,v 1.13 2006/09/29 20:08:20 mortenp Exp $
+ * $Id$
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -36,6 +36,12 @@ char *checkwait_smtpreply(int sockfd, int replytype)
 	char *smtpreply;
 
 	smtpreply = mygetline(sockfd);
+	if(smtpreply == NULL) {
+		/* This will never be a valid SMTP response so will always be returned,
+		 * but is more descriptive than an empty string. */
+		smtpreply = mystrdup("none / error / closed connection");
+	}
+
 #if 0
 	printf("replytype = [%d], smtpreply = [%s]\n", replytype, smtpreply);
 	fprintf(stderr, "%s", smtpreply);
