@@ -1,6 +1,6 @@
 /* Copyright (C) 2003 Mads Martin Joergensen <mmj at mmj.dk>
  *
- * $Id: listcontrol.c,v 1.54 2009/04/11 08:27:36 mmj Exp $
+ * $Id$
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -36,6 +36,7 @@
 #include "find_email_adr.h"
 #include "getlistdelim.h"
 #include "strgen.h"
+#include "prepstdreply.h"
 #include "send_help.h"
 #include "send_list.h"
 #include "log_error.h"
@@ -50,9 +51,11 @@
 enum ctrl_e {
 	CTRL_SUBSCRIBE_DIGEST,
 	CTRL_SUBSCRIBE_NOMAIL,
+	CTRL_SUBSCRIBE_BOTH,
 	CTRL_SUBSCRIBE,
 	CTRL_CONFSUB_DIGEST,
 	CTRL_CONFSUB_NOMAIL,
+	CTRL_CONFSUB_BOTH,
 	CTRL_CONFSUB,
 	CTRL_UNSUBSCRIBE_DIGEST,
 	CTRL_UNSUBSCRIBE_NOMAIL,
@@ -61,6 +64,10 @@ enum ctrl_e {
 	CTRL_CONFUNSUB_NOMAIL,
 	CTRL_CONFUNSUB,
 	CTRL_BOUNCES,
+	CTRL_RELEASE,
+	CTRL_REJECT,
+	CTRL_PERMIT,
+	CTRL_OBSTRUCT,
 	CTRL_MODERATE,
 	CTRL_HELP,
 	CTRL_FAQ,
@@ -80,9 +87,11 @@ struct ctrl_command {
 static struct ctrl_command ctrl_commands[] = {
 	{ "subscribe-digest",   0 },
 	{ "subscribe-nomail",   0 },
+	{ "subscribe-both",     0 },
 	{ "subscribe",          0 },
 	{ "confsub-digest",     1 },
 	{ "confsub-nomail",     1 },
+	{ "confsub-both",       1 },
 	{ "confsub",            1 },
 	{ "unsubscribe-digest", 0 },
 	{ "unsubscribe-nomail", 0 },
@@ -91,6 +100,10 @@ static struct ctrl_command ctrl_commands[] = {
 	{ "confunsub-nomail",   1 },
 	{ "confunsub",          1 },
 	{ "bounces",            1 },
+	{ "release",            1 },
+	{ "reject",             1 },
+	{ "permit",             1 },
+	{ "obstruct",           1 },
 	{ "moderate",           1 },
 	{ "help",               0 },
 	{ "faq",                0 },
@@ -105,9 +118,10 @@ int listcontrol(struct email_container *fromemails, const char *listdir,
 		const char *mlmmjbounce, const char *mailname)
 {
 	char *bouncenr, *tmpstr;
-	char *param = NULL, *conffilename, *moderatefilename;
+	char *param = NULL, *conffilename, *moderatefilename, *gatekeepfilename;
+	char *omitfilename;
+	char *omit = NULL;
 	char *c, *archivefilename, *sendfilename;
-	const char *subswitch;
 	struct stat stbuf;
 	int closedlist, nosubconfirm, tmpfd, noget, i, closedlistsub,
 	    subonlyget = 0;
@@ -115,6 +129,8 @@ int listcontrol(struct email_container *fromemails, const char *listdir,
 	unsigned int ctrl;
 	struct strlist *owners;
 	int owner_idx;
+	text *txt;
+	char *queuefilename;
 	
 	/* A closed list doesn't allow subscribtion and unsubscription */
 	closedlist = statctrl(listdir, "closedlist");
@@ -123,10 +139,6 @@ int listcontrol(struct email_container *fromemails, const char *listdir,
 	closedlistsub = statctrl(listdir, "closedlistsub");
 
 	nosubconfirm = statctrl(listdir, "nosubconfirm");
-	if(nosubconfirm)
-		subswitch = "-c";
-	else
-		subswitch = "-C";
 	
 #if 0
 	log_error(LOG_ARGS, "controlstr = [%s]\n", controlstr);
@@ -208,10 +220,20 @@ int listcontrol(struct email_container *fromemails, const char *listdir,
 		}
 		if (statctrl(listdir, "nodigestsub")) {
 			errno = 0;
-			log_error(LOG_ARGS, "A subcribe-digest request was"
+			log_error(LOG_ARGS, "A subscribe-digest request was"
 				" denied");
-			send_help(listdir, fromemails->emaillist[0],
-				mlmmjsend, "nodigest", "sub-deny-digest");
+			txt = open_text(listdir, "deny", "sub", "disabled",
+					"digest", "sub-deny-digest");
+			MY_ASSERT(txt);
+			register_unformatted(txt, "subaddr",
+					fromemails->emaillist[0]);
+			queuefilename = prepstdreply(txt, listdir,
+					"$listowner$",
+					fromemails->emaillist[0], NULL);
+			MY_ASSERT(queuefilename);
+			close_text(txt);
+			send_help(listdir, queuefilename,
+					fromemails->emaillist[0], mlmmjsend);
 			return -1;
 		}
 		log_oper(listdir, OPLOGFNAME, "mlmmj-sub: request for digest"
@@ -221,7 +243,9 @@ int listcontrol(struct email_container *fromemails, const char *listdir,
 				"-L", listdir,
 				"-a", fromemails->emaillist[0],
 				"-d",
-				subswitch, (char *)NULL);
+				"-r", "-c",
+				(nosubconfirm ? (char *)NULL : "-C"),
+				(char *)NULL);
 		log_error(LOG_ARGS, "execlp() of '%s' failed",
 					mlmmjsub);
 		exit(EXIT_FAILURE);
@@ -245,10 +269,20 @@ int listcontrol(struct email_container *fromemails, const char *listdir,
 		}
 		if (statctrl(listdir, "nonomailsub")) {
 			errno = 0;
-			log_error(LOG_ARGS, "A subcribe-nomail request was"
+			log_error(LOG_ARGS, "A subscribe-nomail request was"
 				" denied");
-			send_help(listdir, fromemails->emaillist[0],
-				mlmmjsend, "nonomail", "sub-deny-nomail");
+			txt = open_text(listdir, "deny", "sub", "disabled",
+					"nomail", "sub-deny-nomail");
+			MY_ASSERT(txt);
+			register_unformatted(txt, "subaddr",
+					fromemails->emaillist[0]);
+			queuefilename = prepstdreply(txt, listdir,
+					"$listowner$",
+					fromemails->emaillist[0], NULL);
+			MY_ASSERT(queuefilename);
+			close_text(txt);
+			send_help(listdir, queuefilename,
+					fromemails->emaillist[0], mlmmjsend);
 			return -1;
 		}
 		log_oper(listdir, OPLOGFNAME, "mlmmj-sub: request for nomail"
@@ -258,7 +292,58 @@ int listcontrol(struct email_container *fromemails, const char *listdir,
 				"-L", listdir,
 				"-a", fromemails->emaillist[0],
 				"-n",
-				subswitch, (char *)NULL);
+				"-r", "-c",
+				(nosubconfirm ? (char *)NULL : "-C"),
+				(char *)NULL);
+		log_error(LOG_ARGS, "execlp() of '%s' failed",
+					mlmmjsub);
+		exit(EXIT_FAILURE);
+		break;
+
+	/* listname+subscribe-both@domain.tld */
+	case CTRL_SUBSCRIBE_BOTH:
+		if (closedlist || closedlistsub) {
+			errno = 0;
+			log_error(LOG_ARGS, "A subscribe-both request was"
+				" sent to a closed list. Ignoring mail");
+			return -1;
+		}
+		if (!strchr(fromemails->emaillist[0], '@')) {
+			/* Not a valid From: address */
+			errno = 0;
+			log_error(LOG_ARGS, "A subscribe-both request was"
+				" sent with an invalid From: header."
+				" Ignoring mail");
+			return -1;
+		}
+		if (statctrl(listdir, "nodigestsub")) {
+			errno = 0;
+			log_error(LOG_ARGS, "A subscribe-both request was"
+				" denied");
+			txt = open_text(listdir, "deny", "sub", "disabled",
+					"both", "sub-deny-digest");
+			MY_ASSERT(txt);
+			register_unformatted(txt, "subaddr",
+					fromemails->emaillist[0]);
+			queuefilename = prepstdreply(txt, listdir,
+					"$listowner$",
+					fromemails->emaillist[0], NULL);
+			MY_ASSERT(queuefilename);
+			close_text(txt);
+			send_help(listdir, queuefilename,
+					fromemails->emaillist[0], mlmmjsend);
+			return -1;
+		}
+		log_oper(listdir, OPLOGFNAME, "mlmmj-sub: request for both"
+					" subscription from %s",
+					fromemails->emaillist[0]);
+		execlp(mlmmjsub, mlmmjsub,
+				"-L", listdir,
+				"-a", fromemails->emaillist[0],
+				"-b",
+				"-r", "-c",
+				(nosubconfirm ? (char *)NULL : "-C"),
+				(char *)NULL);
 		log_error(LOG_ARGS, "execlp() of '%s' failed",
 					mlmmjsub);
 		exit(EXIT_FAILURE);
@@ -286,7 +371,9 @@ int listcontrol(struct email_container *fromemails, const char *listdir,
 		execlp(mlmmjsub, mlmmjsub,
 				"-L", listdir,
 				"-a", fromemails->emaillist[0],
-				subswitch, (char *)NULL);
+				"-r", "-c",
+				(nosubconfirm ? (char *)NULL : "-C"),
+				(char *)NULL);
 		log_error(LOG_ARGS, "execlp() of '%s' failed",
 					mlmmjsub);
 		exit(EXIT_FAILURE);
@@ -314,7 +401,7 @@ int listcontrol(struct email_container *fromemails, const char *listdir,
 				"-L", listdir,
 				"-a", tmpstr,
 				"-d",
-				"-c", (char *)NULL);
+				"-R", "-c", (char *)NULL);
 		log_error(LOG_ARGS, "execlp() of '%s' failed",
 				mlmmjsub);
 		exit(EXIT_FAILURE);
@@ -342,7 +429,35 @@ int listcontrol(struct email_container *fromemails, const char *listdir,
 				"-L", listdir,
 				"-a", tmpstr,
 				"-n",
-				"-c", (char *)NULL);
+				"-R", "-c", (char *)NULL);
+		log_error(LOG_ARGS, "execlp() of '%s' failed",
+				mlmmjsub);
+		exit(EXIT_FAILURE);
+		break;
+
+	/* listname+subconf-both-COOKIE@domain.tld */
+	case CTRL_CONFSUB_BOTH:
+		conffilename = concatstr(3, listdir, "/subconf/", param);
+		myfree(param);
+		if((tmpfd = open(conffilename, O_RDONLY)) < 0) {
+			/* invalid COOKIE */
+			errno = 0;
+			log_error(LOG_ARGS, "A subconf-both request was"
+				" sent with a mismatching cookie."
+				" Ignoring mail");
+			return -1;
+		}
+		tmpstr = mygetline(tmpfd);
+		chomp(tmpstr);
+		close(tmpfd);
+		unlink(conffilename);
+		log_oper(listdir, OPLOGFNAME, "mlmmj-sub: %s confirmed"
+					" subscription to both", tmpstr);
+		execlp(mlmmjsub, mlmmjsub,
+				"-L", listdir,
+				"-a", tmpstr,
+				"-b",
+				"-R", "-c", (char *)NULL);
 		log_error(LOG_ARGS, "execlp() of '%s' failed",
 				mlmmjsub);
 		exit(EXIT_FAILURE);
@@ -370,70 +485,16 @@ int listcontrol(struct email_container *fromemails, const char *listdir,
 		execlp(mlmmjsub, mlmmjsub,
 				"-L", listdir,
 				"-a", tmpstr,
-				"-c", (char *)NULL);
+				"-R", "-c", (char *)NULL);
 		log_error(LOG_ARGS, "execlp() of '%s' failed",
 				mlmmjsub);
 		exit(EXIT_FAILURE);
 		break;
 
-	/* listname+unsubscribe-digest@domain.tld */
+	/* DEPRECATED: listname+unsubscribe-digest@domain.tld */
 	case CTRL_UNSUBSCRIBE_DIGEST:
-		if (closedlist) {
-			errno = 0;
-			log_error(LOG_ARGS, "An unsubscribe-digest request was"
-				" sent to a closed list. Ignoring mail");
-			return -1;
-		}
-		if (!strchr(fromemails->emaillist[0], '@')) {
-			/* Not a valid From: address */
-			errno = 0;
-			log_error(LOG_ARGS, "An unsubscribe-digest request was"
-				" sent with an invalid From: header."
-				" Ignoring mail");
-			return -1;
-		}
-		log_oper(listdir, OPLOGFNAME, "mlmmj-unsub: %s requests"
-					" unsubscribe from digest",
-					fromemails->emaillist[0]);
-		execlp(mlmmjunsub, mlmmjunsub,
-				"-L", listdir,
-				"-a", fromemails->emaillist[0],
-				"-d",
-				subswitch, (char *)NULL);
-		log_error(LOG_ARGS, "execlp() of '%s' failed",
-				mlmmjunsub);
-		exit(EXIT_FAILURE);
-		break;
-
-	/* listname+unsubscribe-nomail@domain.tld */
+	/* DEPRECATED: listname+unsubscribe-nomail@domain.tld */
 	case CTRL_UNSUBSCRIBE_NOMAIL:
-		if (closedlist) {
-			errno = 0;
-			log_error(LOG_ARGS, "An unsubscribe-nomail request was"
-				" sent to a closed list. Ignoring mail");
-			return -1;
-		}
-		if (!strchr(fromemails->emaillist[0], '@')) {
-			/* Not a valid From: address */
-			errno = 0;
-			log_error(LOG_ARGS, "An unsubscribe-nomail request was"
-				" sent with an invalid From: header."
-				" Ignoring mail");
-			return -1;
-		}
-		log_oper(listdir, OPLOGFNAME, "mlmmj-unsub: %s requests"
-					" unsubscribe from nomail",
-					fromemails->emaillist[0]);
-		execlp(mlmmjunsub, mlmmjunsub,
-				"-L", listdir,
-				"-a", fromemails->emaillist[0],
-				"-n",
-				subswitch, (char *)NULL);
-		log_error(LOG_ARGS, "execlp() of '%s' failed",
-				mlmmjunsub);
-		exit(EXIT_FAILURE);
-		break;
-
 	/* listname+unsubscribe@domain.tld */
 	case CTRL_UNSUBSCRIBE:
 		if (closedlist) {
@@ -451,12 +512,13 @@ int listcontrol(struct email_container *fromemails, const char *listdir,
 			return -1;
 		}
 		log_oper(listdir, OPLOGFNAME, "mlmmj-unsub: %s requests"
-					" unsubscribe from regular list",
+					" unsubscribe",
 					fromemails->emaillist[0]);
 		execlp(mlmmjunsub, mlmmjunsub,
 				"-L", listdir,
 				"-a", fromemails->emaillist[0],
-				subswitch, (char *)NULL);
+				"-r", (nosubconfirm ? "-c" : "-C"),
+				(char *)NULL);
 		log_error(LOG_ARGS, "execlp() of '%s' failed",
 				mlmmjunsub);
 		exit(EXIT_FAILURE);
@@ -490,7 +552,7 @@ int listcontrol(struct email_container *fromemails, const char *listdir,
 				"-L", listdir,
 				"-a", tmpstr,
 				"-d",
-				"-c", (char *)NULL);
+				"-R", "-c", (char *)NULL);
 		log_error(LOG_ARGS, "execlp() of '%s' failed",
 				mlmmjunsub);
 		exit(EXIT_FAILURE);
@@ -524,7 +586,7 @@ int listcontrol(struct email_container *fromemails, const char *listdir,
 				"-L", listdir,
 				"-a", tmpstr,
 				"-n",
-				"-c", (char *)NULL);
+				"-R", "-c", (char *)NULL);
 		log_error(LOG_ARGS, "execlp() of '%s' failed",
 				mlmmjunsub);
 		exit(EXIT_FAILURE);
@@ -558,7 +620,7 @@ int listcontrol(struct email_container *fromemails, const char *listdir,
 		execlp(mlmmjunsub, mlmmjunsub,
 				"-L", listdir,
 				"-a", tmpstr,
-				"-c", (char *)NULL);
+				"-R", "-c", (char *)NULL);
 		log_error(LOG_ARGS, "execlp() of '%s' failed",
 				mlmmjunsub);
 		exit(EXIT_FAILURE);
@@ -588,49 +650,145 @@ int listcontrol(struct email_container *fromemails, const char *listdir,
 		exit(EXIT_FAILURE);
 		break;
 
-	/* listname+moderate-COOKIE@domain.tld */
+	/* listname+release-COOKIE@domain.tld */
+	case CTRL_RELEASE:
+	/* DEPRECATED: listname+moderate-COOKIE@domain.tld */
+	/* DEPRECATED: listname+moderate-subscribeCOOKIE@domain.tld */
 	case CTRL_MODERATE:
-		/* TODO Add accept/reject parameter to moderate */
-		moderatefilename = concatstr(3, listdir, "/moderation/", param);
-
-		/* Subscriber moderation */
+		/* Subscriber moderation; DEPRECATED */
 		if(strncmp(param, "subscribe", 9) == 0) {
-			log_oper(listdir, OPLOGFNAME, "%s moderated %s",
-				fromemails->emaillist[0], moderatefilename);
-			execlp(mlmmjsub, mlmmjsub,
-					"-L", listdir,
-					"-m", param,
-					"-c", (char *)NULL);
+			tmpstr = mystrdup(param + 9);
+			myfree(param);
+			param = tmpstr;
+			goto permit;
 		}
 
-		sendfilename = concatstr(2, moderatefilename, ".sending");
-
+		moderatefilename = concatstr(3, listdir, "/moderation/", param);
 		if(stat(moderatefilename, &stbuf) < 0) {
 			myfree(moderatefilename);
 			/* no mail to moderate */
 			errno = 0;
-			log_error(LOG_ARGS, "A moderate request was"
+			log_error(LOG_ARGS, "A release request was"
 				" sent with a mismatching cookie."
 				" Ignoring mail");
 			return -1;
 		}
+
+		sendfilename = concatstr(2, moderatefilename, ".sending");
 		/* Rename it to avoid mail being sent twice */
 		if(rename(moderatefilename, sendfilename) < 0) {
 			log_error(LOG_ARGS, "Could not rename to .sending");
 			exit(EXIT_FAILURE);
 		}
 
-		log_oper(listdir, OPLOGFNAME, "%s moderated %s",
-				fromemails->emaillist[0], moderatefilename);
+		omitfilename = concatstr(2, moderatefilename, ".omit");
+		if(stat(omitfilename, &stbuf) == 0) {
+			tmpfd = open(omitfilename, O_RDONLY);
+			if(tmpfd < 0) {
+				log_error(LOG_ARGS, "Could not open %s",
+						omitfilename);
+			} else {
+				omit = mygetline(tmpfd);
+				close(tmpfd);
+				chomp(omit);
+			}
+			unlink(omitfilename);
+			myfree(omitfilename);
+		}
+
 		myfree(moderatefilename);
-		execlp(mlmmjsend, mlmmjsend,
-				"-L", listdir,
-				"-m", sendfilename, (char *)NULL);
+
+		log_oper(listdir, OPLOGFNAME, "%s released %s",
+				fromemails->emaillist[0], param);
+
+		if (omit != NULL)
+			execlp(mlmmjsend, mlmmjsend,
+					"-L", listdir,
+					"-o", omit,
+					"-m", sendfilename, (char *)NULL);
+		else
+			execlp(mlmmjsend, mlmmjsend,
+					"-L", listdir,
+					"-m", sendfilename, (char *)NULL);
 		log_error(LOG_ARGS, "execlp() of '%s' failed",
 					mlmmjsend);
 		exit(EXIT_FAILURE);
 		break;
 
+	/* listname+reject-COOKIE@domain.tld */
+	case CTRL_REJECT:
+		moderatefilename = concatstr(3, listdir, "/moderation/", param);
+		if(stat(moderatefilename, &stbuf) < 0) {
+			myfree(moderatefilename);
+			/* no mail to moderate */
+			errno = 0;
+			log_error(LOG_ARGS, "A reject request was"
+				" sent with a mismatching cookie."
+				" Ignoring mail");
+			return -1;
+		}
+		log_oper(listdir, OPLOGFNAME, "%s rejected %s",
+			fromemails->emaillist[0], param);
+		myfree(param);
+		if (unlink(moderatefilename) != 0) {
+			log_error(LOG_ARGS, "Could not unlink %s",
+					moderatefilename);
+			myfree(moderatefilename);
+			exit(EXIT_FAILURE);
+		}
+		myfree(moderatefilename);
+		break;
+
+	/* listname+permit-COOKIE@domain.tld */
+	case CTRL_PERMIT:
+permit:
+		gatekeepfilename = concatstr(3, listdir,
+				"/moderation/subscribe", param);
+		if(stat(gatekeepfilename, &stbuf) < 0) {
+			myfree(gatekeepfilename);
+			/* no mail to moderate */
+			errno = 0;
+			log_error(LOG_ARGS, "A permit request was"
+				" sent with a mismatching cookie."
+				" Ignoring mail");
+			return -1;
+		}
+		log_oper(listdir, OPLOGFNAME, "%s permitted %s",
+			fromemails->emaillist[0], param);
+		execlp(mlmmjsub, mlmmjsub,
+				"-L", listdir,
+				"-m", param,
+				"-c", (char *)NULL);
+		log_error(LOG_ARGS, "execlp() of '%s' failed",
+					mlmmjsub);
+		exit(EXIT_FAILURE);
+		break;
+
+	/* listname+obstruct-COOKIE@domain.tld */
+	case CTRL_OBSTRUCT:
+		gatekeepfilename = concatstr(3, listdir,
+				"/moderation/subscribe", param);
+		if(stat(gatekeepfilename, &stbuf) < 0) {
+			myfree(gatekeepfilename);
+			/* no mail to moderate */
+			errno = 0;
+			log_error(LOG_ARGS, "An obstruct request was"
+				" sent with a mismatching cookie."
+				" Ignoring mail");
+			return -1;
+		}
+		log_oper(listdir, OPLOGFNAME, "%s obstructed %s",
+			fromemails->emaillist[0], param);
+		myfree(param);
+		if (unlink(gatekeepfilename) != 0) {
+			log_error(LOG_ARGS, "Could not unlink %s",
+					gatekeepfilename);
+			myfree(gatekeepfilename);
+			exit(EXIT_FAILURE);
+		}
+		myfree(gatekeepfilename);
+		break;
+
 	/* listname+help@domain.tld */
 	case CTRL_HELP:
 		if(!strchr(fromemails->emaillist[0], '@')) {
@@ -643,7 +801,14 @@ int listcontrol(struct email_container *fromemails, const char *listdir,
 		}
 		log_oper(listdir, OPLOGFNAME, "%s requested help",
 				fromemails->emaillist[0]);
-		send_help(listdir, fromemails->emaillist[0], mlmmjsend, "help", "listhelp");
+		txt = open_text(listdir, "help", NULL, NULL, NULL, "listhelp");
+		MY_ASSERT(txt);
+		queuefilename = prepstdreply(txt, listdir,
+				"$listowner$", fromemails->emaillist[0], NULL);
+		MY_ASSERT(queuefilename);
+		close_text(txt);
+		send_help(listdir, queuefilename,
+				fromemails->emaillist[0], mlmmjsend);
 		break;
 
        /* listname+faq@domain.tld */
@@ -658,7 +823,14 @@ int listcontrol(struct email_container *fromemails, const char *listdir,
                }
                log_oper(listdir, OPLOGFNAME, "%s requested faq",
                                fromemails->emaillist[0]);
-               send_help(listdir, fromemails->emaillist[0], mlmmjsend, "faq", "listfaq");
+		txt = open_text(listdir, "faq", NULL, NULL, NULL, "listfaq");
+		MY_ASSERT(txt);
+		queuefilename = prepstdreply(txt, listdir,
+				"$listowner$", fromemails->emaillist[0], NULL);
+		MY_ASSERT(queuefilename);
+		close_text(txt);
+		send_help(listdir, queuefilename,
+				fromemails->emaillist[0], mlmmjsend);
                break;
 
 	/* listname+get-INDEX@domain.tld */
@@ -672,7 +844,8 @@ int listcontrol(struct email_container *fromemails, const char *listdir,
 		}
 		subonlyget = statctrl(listdir, "subonlyget");
 		if(subonlyget) {
-			if(is_subbed(listdir, fromemails->emaillist[0]) != 0) {
+			if(is_subbed(listdir, fromemails->emaillist[0], 0) ==
+					SUB_NONE) {
 				errno = 0;
 				log_error(LOG_ARGS, "A get request was sent"
 					" from a non-subscribed address to a"
