@@ -1,6 +1,6 @@
 /* Copyright (C) 2004, 2005 Morten K. Poulsen <morten at afdelingp.dk>
  *
- * $Id: send_digest.c,v 1.14 2007/11/05 21:23:37 mortenp Exp $
+ * $Id$
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -59,20 +59,57 @@ struct thread {
 };
 
 
-static char *thread_list(const char *listdir, int firstindex, int lastindex)
+struct thread_list_state;
+typedef struct thread_list_state thread_list_state;
+struct thread_list_state {
+	const char *listdir;
+	int firstindex;
+	int lastindex;
+	int num_threads;
+	struct thread *threads;
+	int cur_thread;
+	int cur_mail;
+};
+
+
+static thread_list_state *init_thread_list(
+		const char *listdir, int firstindex, int lastindex)
 {
+	/* We use a static variable rather than dynamic allocation as
+	 * there will never be two lists in use simultaneously */
+	static thread_list_state s;
+	s.listdir = listdir;
+	s.firstindex = firstindex;
+	s.lastindex = lastindex;
+	s.num_threads = 0;
+	s.threads = NULL;
+	s.cur_thread = -1;
+	return &s;
+}
+
+
+static void rewind_thread_list(void * state)
+{
+	thread_list_state *s = (thread_list_state *)state;
 	int i, j, archivefd, thread_idx;
-	char *ret, *line, *tmp, *subj, *from;
+	char *line, *tmp, *subj, *from;
 	char *archivename;
 	int num_threads = 0;
 	struct thread *threads = NULL;
 	char buf[45];
 
-	for (i=firstindex; i<=lastindex; i++) {
+	if (s->cur_thread != -1) {
+		/* We have gathered the data already; just rewind */
+		s->cur_thread = 0;
+		s->cur_mail = -1;
+		return;
+	}
+
+	for (i=s->firstindex; i<=s->lastindex; i++) {
 
 		snprintf(buf, sizeof(buf), "%d", i);
 
-		archivename = concatstr(3, listdir, "/archive/", buf);
+		archivename = concatstr(3, s->listdir, "/archive/", buf);
 		archivefd = open(archivename, O_RDONLY);
 		myfree(archivename);
 
@@ -131,7 +168,7 @@ static char *thread_list(const char *listdir, int firstindex, int lastindex)
 			num_threads++;
 			threads = myrealloc(threads,
 					num_threads*sizeof(struct thread));
-			threads[num_threads-1].subject = mystrdup(tmp);
+			threads[num_threads-1].subject = concatstr(2,tmp,"\n");
 			threads[num_threads-1].num_mails = 0;
 			threads[num_threads-1].mails = NULL;
 			thread_idx = num_threads-1;
@@ -150,43 +187,61 @@ static char *thread_list(const char *listdir, int firstindex, int lastindex)
 		close(archivefd);
 	}
 
-	ret = mystrdup("");
+	s->num_threads = num_threads;
+	s->threads = threads;
+	s->cur_thread = 0;
+	s->cur_mail = -1;
+}
 
-	for (i=0; i<num_threads; i++) {
 
-		tmp = concatstr(3, ret, threads[i].subject, "\n");
-		myfree(ret);
-		ret = tmp;
-		myfree(threads[i].subject);
+static const char *get_thread_list_line(void * state)
+{
+	thread_list_state *s = (thread_list_state *)state;
 
-		for (j=0; j<threads[i].num_mails; j++) {
-			tmp = concatstr(2, ret, threads[i].mails[j].from);
-			myfree(ret);
-			ret = tmp;
-			myfree(threads[i].mails[j].from);
-		}
-		myfree(threads[i].mails);
+	if (s->cur_thread >= s->num_threads) return NULL;
 
-		tmp = concatstr(2, ret, "\n");
-		myfree(ret);
-		ret = tmp;
+	if (s->cur_mail == -1) {
+		s->cur_mail = 0;
+		return s->threads[s->cur_thread].subject;
 	}
-	myfree(threads);
 
-	return ret;
+	if (s->cur_mail >= s->threads[s->cur_thread].num_mails) {
+		s->cur_thread++;
+		s->cur_mail = -1;
+		return "\n";
+	}
+
+	return s->threads[s->cur_thread].mails[s->cur_mail++].from;
+}
+
+
+static void finish_thread_list(thread_list_state * s)
+{
+	int i, j;
+	if (s->threads == NULL) return;
+	for (i=0; i<s->num_threads; i++) {
+		myfree(s->threads[i].subject);
+		for (j=0; j<s->threads[i].num_mails; j++) {
+			myfree(s->threads[i].mails[j].from);
+		}
+		myfree(s->threads[i].mails);
+	}
+	myfree(s->threads);
+	s->threads = NULL;
 }
 
 
 int send_digest(const char *listdir, int firstindex, int lastindex,
 		int issue, const char *addr, const char *mlmmjsend)
 {
-	int i, fd, archivefd, status, hdrfd, txtfd;
-	char buf[45];
-	char *tmp, *queuename = NULL, *archivename, *subject, *line = NULL;
-	char *utfsub, *utfsub2, *utfline;
+	int i, fd, archivefd, status, hdrfd;
+	size_t len;
+	text * txt;
+	char buf[100];
+	char *tmp, *queuename = NULL, *archivename, *subject = NULL, *line = NULL;
 	char *boundary, *listaddr, *listdelim, *listname, *listfqdn;
-	char *subst_data[10];
 	pid_t childpid, pid;
+	thread_list_state * tls;
 
 	if (addr) {
 		errno = 0;
@@ -224,49 +279,90 @@ int send_digest(const char *listdir, int firstindex, int lastindex,
 	listfqdn = genlistfqdn(listaddr);
 	listdelim = getlistdelim(listdir);
 	
-	txtfd = open_listtext(listdir, "digest");
-	if (txtfd < 0) {
-		log_error(LOG_ARGS, "Notice: Could not open std mail digest");
+	txt = open_text_file(listdir, "digest");
+	if (txt == NULL) {
+		log_error(LOG_ARGS, "Could not open listtext 'digest'");
+		goto fallback_subject;
 	}
 
-	subst_data[0] = "digestfirst";
 	snprintf(buf, sizeof(buf), "%d", firstindex);
-	subst_data[1] = mystrdup(buf);
+	register_unformatted(txt, "digestfirst", buf);
 
-	subst_data[2] = "digestlast";
 	snprintf(buf, sizeof(buf), "%d", lastindex);
-	subst_data[3] = mystrdup(buf);
+	register_unformatted(txt, "digestlast", buf);
 
-	subst_data[4] = "digestinterval";
 	if (lastindex == firstindex) {
 		snprintf(buf, sizeof(buf), "%d", firstindex);
 	} else {
 		snprintf(buf, sizeof(buf), "%d-%d", firstindex, lastindex);
 	}
-	subst_data[5] = mystrdup(buf);
+	register_unformatted(txt, "digestinterval", buf);
 
-	subst_data[6] = "digestissue";
 	snprintf(buf, sizeof(buf), "%d", issue);
-	subst_data[7] = mystrdup(buf);
+	register_unformatted(txt, "digestissue", buf);
 
-	subst_data[8] = "digestthreads";
-	subst_data[9] = thread_list(listdir, firstindex, lastindex);
+	register_unformatted(txt, "digestthreads", "%digestthreads%"); /* DEPRECATED */
 
-	if ((txtfd < 0) || !(line = mygetline(txtfd)) ||
-			(strncasecmp(line, "Subject: ", 9) != 0)) {
+	tls = init_thread_list(listdir, firstindex, lastindex);
+	register_formatted(txt, "digestthreads", rewind_thread_list,
+			get_thread_list_line, tls);
 
-		utfsub = mystrdup("Digest of $listaddr$ issue $digestissue$"
-				" ($digestinterval$)");
-	} else {
+	line = get_processed_text_line(txt, 1, listaddr, listdelim, listdir);
 
-		chomp(line);
-		utfsub = unistr_escaped_to_utf8(line + 9);
+	if (line == NULL) {
+		log_error(LOG_ARGS, "No content in digest listtext");
+		goto fallback_subject;
 	}
 
-	utfsub2 = substitute(utfsub, listaddr, listdelim, 5, subst_data, NULL);
-	subject = unistr_utf8_to_header(utfsub2);
-	myfree(utfsub);
-	myfree(utfsub2);
+	tmp = line;
+	len = 0;
+	while (*tmp && *tmp != ':') {
+		tmp++;
+		len++;
+	}
+	if (!*tmp) {
+		log_error(LOG_ARGS, "No subject or invalid "
+				"subject in digest listtext");
+		goto fallback_subject;
+	}
+	tmp++;
+	len++;
+	if (strncasecmp(line, "Subject:", len) == 0) {
+		tmp = unistr_utf8_to_header(tmp);
+		subject = concatstr(2, "Subject:", tmp);
+		myfree(tmp);
+		myfree(line);
+
+		/* Skip the empty line after the subject */
+		line = get_processed_text_line(txt, 1, listaddr, listdelim,
+				listdir);
+		if (line == NULL || *line != '\0') {
+			log_error(LOG_ARGS, "Too many headers "
+					"in digest listtext");
+			goto fallback_subject;
+		}
+
+		if (line != NULL) myfree(line);
+		line = NULL;
+	} else {
+		log_error(LOG_ARGS, "No subject or invalid "
+				"subject in digest listtext");
+		goto fallback_subject;
+	}
+
+fallback_subject:
+	if (subject == NULL) {
+		if (lastindex == firstindex) {
+			snprintf(buf, sizeof(buf), "%d", firstindex);
+		} else {
+			snprintf(buf, sizeof(buf), "%d-%d", firstindex, lastindex);
+		}
+		tmp = mystrdup(buf);
+		snprintf(buf, sizeof(buf), "Digest of %s issue %d (%s)",
+				listaddr, issue, tmp);
+		subject = unistr_utf8_to_header(buf);
+		myfree(tmp);
+	}
 
 	tmp = concatstr(10, "From: ", listname, listdelim, "help@", listfqdn,
 			   "\nMIME-Version: 1.0"
@@ -302,13 +398,8 @@ errdighdrs:
 		myfree(listaddr);
 		myfree(listname);
 		myfree(listdelim);
-		myfree(subst_data[1]);
-		myfree(subst_data[3]);
-		myfree(subst_data[5]);
-		myfree(subst_data[7]);
-		myfree(subst_data[9]);
-		if (txtfd > 0) {
-			close(txtfd);
+		if (txt != NULL) {
+			close_text(txt);
 			myfree(line);
 		}
 		if (hdrfd > 0) {
@@ -317,11 +408,11 @@ errdighdrs:
 		return -1;
 	}
 
-	if ((txtfd > 0) && !statctrl(listdir, "nodigesttext")) {
+	if ((txt != NULL) && !statctrl(listdir, "nodigesttext")) {
 
 		tmp = concatstr(3, "\n--", boundary,
 				"\nContent-Type: text/plain; charset=UTF-8"
-				"\nContent-Encoding: 8bit"
+				"\nContent-Transfer-Encoding: 8bit"
 				"\n\n");
 		if (writen(fd, tmp, strlen(tmp)) == -1) {
 			log_error(LOG_ARGS, "Could not write digest text/plain"
@@ -334,58 +425,39 @@ errdighdrs:
 			myfree(listaddr);
 			myfree(listname);
 			myfree(listdelim);
-			myfree(subst_data[1]);
-			myfree(subst_data[3]);
-			myfree(subst_data[5]);
-			myfree(subst_data[7]);
-			myfree(subst_data[9]);
-			if (txtfd > 0) {
-				close(txtfd);
+			if (txt != NULL) {
+				close_text(txt);
 				myfree(line);
 			}
 			return -1;
 		}
 		myfree(tmp);
 
-		if (line && (strncasecmp(line, "Subject: ", 9) == 0)) {
-			myfree(line);
-			line = mygetline(txtfd);
-			if (line && (strcmp(line, "\n") == 0)) {
-				/* skip empty line after Subject: */
-				line[0] = '\0';
-			}
-		}
-
-		if (line) {
-			do {
-				utfline = unistr_escaped_to_utf8(line);
+		for (;;) {
+			line = get_processed_text_line(txt, 0, listaddr, listdelim,
+					listdir);
+			if (line == NULL) break;
+			len = strlen(line);
+			line[len] = '\n';
+			if(writen(fd, line, len+1) < 0) {
 				myfree(line);
-
-				tmp = substitute(utfline, listaddr, listdelim,
-						5, subst_data, NULL);
-				myfree(utfline);
-
-				if(writen(fd, tmp, strlen(tmp)) < 0) {
-					myfree(tmp);
-					log_error(LOG_ARGS, "Could not write"
-							" std mail");
-					break;
-				}
-				myfree(tmp);
-			} while ((line = mygetline(txtfd)));
+				log_error(LOG_ARGS, "Could not write"
+						" std mail");
+				break;
+			}
+			myfree(line);
 		}
 
-		close(txtfd);
+		finish_thread_list(tls);
+		close_text(txt);
+	} else if (txt != NULL) {
+		finish_thread_list(tls);
+		close_text(txt);
 	}
 
-	myfree(line);
+	if (line != NULL) myfree(line);
 	myfree(listaddr);
 	myfree(listdelim);
-	myfree(subst_data[1]);
-	myfree(subst_data[3]);
-	myfree(subst_data[5]);
-	myfree(subst_data[7]);
-	myfree(subst_data[9]);
 
 	for (i=firstindex; i<=lastindex; i++) {
 		snprintf(buf, sizeof(buf), "%d", i);
