@@ -1,6 +1,6 @@
 /* Copyright (C) 2003 Mads Martin Joergensen <mmj at mmj.dk>
  *
- * $Id: send_help.c,v 1.22 2007/11/05 21:23:37 mortenp Exp $
+ * $Id$
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -42,10 +42,10 @@
 #include "prepstdreply.h"
 #include "memory.h"
 
-void send_help(const char *listdir, const char *emailaddr,
-	       const char *mlmmjsend, const char *name, const char *textfile)
+void send_help(const char *listdir, const char *queuefilename,
+		const char *emailaddr, const char *mlmmjsend)
 {
-	char *queuefilename, *listaddr, *listdelim, *listname, *listfqdn;
+	char *listaddr, *listdelim, *listname, *listfqdn;
 	char *fromaddr;
 
 	listaddr = getlistaddr(listdir);
@@ -54,18 +54,11 @@ void send_help(const char *listdir, const char *emailaddr,
 	listfqdn = genlistfqdn(listaddr);
 
 	fromaddr = concatstr(4, listname, listdelim, "bounces-help@", listfqdn);
-	myfree(listdelim);
 
-	queuefilename = prepstdreply(listdir, textfile, "$listowner$",
-					emailaddr, NULL, 0, NULL, NULL, NULL);
-	if(queuefilename == NULL) {
-		log_error(LOG_ARGS, "Could not prepare %s mail", name);
-		exit(EXIT_FAILURE);
-	}
-	
-	myfree(listaddr);
 	myfree(listname);
+	myfree(listdelim);
 	myfree(listfqdn);
+	myfree(listaddr);
 
 	execlp(mlmmjsend, mlmmjsend,
 				"-l", "1",
